@@ -100,6 +100,11 @@ class VMBroker:
     def _act(self, provider, method, checkout=False):
         """Perform a general action against a provider's method"""
         provider_inst = provider(**self._kwargs)
+        helpers.emit({
+            "provider": provider_inst.__class__.__name__,
+            "action": method,
+            "arguments": self._kwargs
+        })
         result = getattr(provider_inst, method)(**self._kwargs)
         logger.debug(result)
         if result and checkout:
@@ -116,27 +121,28 @@ class VMBroker:
         :return: List of Host objects
         """
         hosts = []
+        if not self._provider_actions:
+            raise self.BrokerError("Could not determine an appropriate provider")
         for action in self._provider_actions.keys():
             provider, method = PROVIDER_ACTIONS[action]
             logger.info(f"Using provider {provider.__name__} to checkout")
-            host = self._act(provider, method, checkout=True)
+            try:
+                host = self._act(provider, method, checkout=True)
+            except exceptions.ProviderError:
+                host = None
             logger.debug(f"host={host}")
             if host:
                 hosts.append(host)
                 logger.info(f"{host.__class__.__name__}: {host.hostname}")
         return hosts
 
-    def checkout(self, connect=False):
+    def checkout(self):
         """checkout one or more VMs
 
-        :param connect: Boolean whether to establish host ssh connection
-
         :return: Host obj or list of Host objects
         """
         hosts = self._checkout()
-        if connect:
-            for host in hosts:
-                host.connect()
+        helpers.emit(hosts=[host.to_dict() for host in hosts])
         self._hosts.extend(hosts)
         helpers.update_inventory([host.to_dict() for host in hosts])
         return hosts if not len(hosts) == 1 else hosts[0]
@@ -151,6 +157,8 @@ class VMBroker:
                 if key in kwargs:
                     self._provider_actions[key] = action
         self._kwargs.update(kwargs)
+        if not self._provider_actions:
+            raise self.BrokerError("Could not determine an appropriate provider")
         for action, arg in self._provider_actions.items():
             provider, method = PROVIDER_ACTIONS[action]
         logger.info(f"Using provider {provider.__name__} for execution")
@@ -163,7 +171,19 @@ class VMBroker:
             logger.info(f"Querying provider {provider.__name__}")
             self._act(provider, "nick_help", checkout=False)
 
-    def checkin(self, host=None):
+    def _checkin(self, host):
+        logger.info(f"Checking in {host.hostname or host.name}")
+        host.close()
+        try:
+            host.release()
+        except Exception:
+            pass
+        if host in self._hosts:
+            self._hosts.remove(host)
+        helpers.update_inventory(remove=host.hostname)
+        return host
+
+    def checkin(self, sequential=False, host=None):
         """checkin one or more VMs
 
         :param host: can be one of:
@@ -171,24 +191,28 @@ class VMBroker:
             A single host object
             A list of host objects
             A dictionary mapping host types to one or more host objects
+
+        :param sequential: boolean whether to run checkins sequentially
         """
-        if host is None:
-            host = self._hosts
-        logger.debug(host)
-        if isinstance(host, dict):
-            for _host in host.values():
-                self.checkin(_host)
-        elif isinstance(host, list):
-            # reversing over a copy of the list to avoid skipping
-            for _host in host[::-1]:
-                self.checkin(_host)
-        elif host:
-            logger.info(f"Checking in {host.hostname or host.name}")
-            host.close()
-            host.release()
-            if host in self._hosts:
-                self._hosts.remove(host)
-            helpers.update_inventory(remove=host.hostname)
+        # default to hosts listed on the instance
+        hosts = host or self._hosts
+        logger.debug(f'Checkin called with: {hosts}, '
+                     f'running {"sequential" if sequential else "concurrent"}')
+        # normalize the type since the function accepts multiple types
+        if isinstance(hosts, dict):
+            # flatten the lists of hosts from the values of the dict
+            hosts = [host for host_list in hosts.values() for host in host_list]
+        if not isinstance(hosts, list):
+            hosts = [hosts]
+        with ProcessPoolExecutor(max_workers=1 if sequential else len(hosts)) as workers:
+            completed_checkins = as_completed(
+                # reversing over a copy of the list to avoid skipping
+                workers.submit(self._checkin, _host)
+                for _host in hosts[::-1]
+            )
+            for completed in completed_checkins:
+                _host = completed.result()
+                logger.debug(f'Completed checkin process for {_host.hostname or _host.name}')
 
     def extend(self, host=None):
         """extend one or more VMs
@@ -232,27 +256,11 @@ class VMBroker:
         prov_inventory = PROVIDERS[provider](**instance).get_inventory(additional_arg)
         curr_inventory = [
             host["hostname"] or host["name"] for host in helpers.load_inventory()
+            if host["_broker_provider"] == provider
         ]
-        new_hosts = []
-        remove_hosts = curr_inventory[:]
-        for n_host in prov_inventory:
-            name = n_host["hostname"] or n_host["name"]
-            if name in curr_inventory:
-                remove_hosts.remove(name)
-            else:
-                new_hosts.append(n_host)
-        if new_hosts:
-            msg = ", ".join([host["hostname"] or host["name"] for host in new_hosts])
-            logger.info(f"Adding new hosts: {msg}")
-            helpers.update_inventory(add=new_hosts)
-        else:
-            logger.info("No new hosts found")
-        if remove_hosts:
-            msg = ", ".join(remove_hosts)
-            logger.info(f"Removing old hosts: {msg}")
-            helpers.update_inventory(remove=remove_hosts)
+        helpers.update_inventory(add=prov_inventory, remove=curr_inventory)
 
-    def reconstruct_host(self, host_export_data, connect=False):
+    def reconstruct_host(self, host_export_data):
         """reconstruct a host from export data"""
         logger.debug(f"reconstructing host with export: {host_export_data}")
         provider = PROVIDERS.get(host_export_data.get("_broker_provider"))
@@ -265,26 +273,25 @@ class VMBroker:
         host = provider_inst.construct_host(
             provider_params=None, host_classes=self.host_classes, **host_export_data
         )
-        if connect:
-            host.connect()
         return host
 
-    def from_inventory(self, connect=False, filter=None):
+    def from_inventory(self, filter=None):
         """Reconstruct one or more hosts from the local inventory
 
-        :param connect: Boolean - establish ssh connection
-
         :param filter: A broker-spec filter string
         """
         inv_hosts = helpers.load_inventory(filter=filter)
-        return [self.reconstruct_host(inv_host, connect) for inv_host in inv_hosts]
+        return [self.reconstruct_host(inv_host) for inv_host in inv_hosts]
 
     def __enter__(self):
         try:
-            hosts = self.checkout(connect=True)
+            hosts = self.checkout()
             if not hosts:
                 raise Exception("No hosts created during checkout")
-            [host.setup() for host in hosts]
+            if isinstance(hosts, list):
+                [host.setup() for host in hosts]
+            else:
+                hosts.setup()
             return hosts
         except Exception as err:
             self.checkin()
