@@ -19,7 +19,9 @@ class ExceptionHandler(click.Group):
         except Exception as err:
             if not isinstance(err, exceptions.BrokerError):
                 err = exceptions.BrokerError(err)
+            helpers.emit(return_code=err.error_code, error_message=str(err.message))
             sys.exit(err.error_code)
+        helpers.emit(return_code=0)
 
 
 def provider_options(command):
@@ -89,6 +91,14 @@ def populate_providers(click_group):
     is_eager=True,
     expose_value=False,
 )
+@click.option(
+    "--output-file",
+    type=click.Path(dir_okay=False),
+    callback=helpers.set_emit_file,
+    is_eager=True,
+    expose_value=False,
+    help="Path to file where emitted json values should be stored"
+)
 @click.option(
     "--version",
     is_flag=True,
@@ -171,10 +181,11 @@ populate_providers(providers)
 @click.argument("vm", type=str, nargs=-1)
 @click.option("-b", "--background", is_flag=True, help="Run checkin in the background")
 @click.option("--all", "all_", is_flag=True, help="Select all VMs")
+@click.option("--sequential", is_flag=True, default=False, help="Run checkins sequentially")
 @click.option(
     "--filter", type=str, help="Checkin only what matches the specified filter"
 )
-def checkin(vm, background, all_, filter):
+def checkin(vm, background, all_, sequential, filter):
     """Checkin or "remove" a VM or series of VM broker instances
 
     COMMAND: broker checkin <vm hostname>|<local id>|all
@@ -183,6 +194,10 @@ def checkin(vm, background, all_, filter):
 
     :param background: run a new broker subprocess to carry out command
 
+    :param all_: Flag for whether to checkin everything
+
+    :param sequential: Flag for whether to run checkins sequentially
+
     :param filter: a filter string matching broker's specification
     """
     if background:
@@ -193,7 +208,7 @@ def checkin(vm, background, all_, filter):
         if str(num) in vm or host["hostname"] in vm or host["name"] in vm or all_:
             to_remove.append(VMBroker().reconstruct_host(host))
     broker_inst = VMBroker(hosts=to_remove)
-    broker_inst.checkin()
+    broker_inst.checkin(sequential=sequential)
 
 
 @cli.command()
@@ -214,14 +229,16 @@ def inventory(details, sync, filter):
         VMBroker.sync_inventory(provider=sync)
     logger.info("Pulling local inventory")
     inventory = helpers.load_inventory(filter=filter)
+    emit_data = []
     for num, host in enumerate(inventory):
+        emit_data.append(host)
         if details:
             logger.info(
                 f"{num}: {host['hostname'] or host['name']}, Details: {helpers.yaml_format(host)}"
             )
         else:
             logger.info(f"{num}: {host['hostname'] or host['name']}")
-
+    helpers.emit({"inventory": emit_data})
 
 @cli.command()
 @click.argument("vm", type=str, nargs=-1)
@@ -230,7 +247,8 @@ def inventory(details, sync, filter):
 @click.option(
     "--filter", type=str, help="Extend only what matches the specified filter"
 )
-def extend(vm, background, all_, filter):
+@provider_options
+def extend(vm, background, all_, filter, **kwargs):
     """Extend a host's lease time
 
     COMMAND: broker extend <vm hostname>|<vm name>|<local id>
@@ -243,6 +261,7 @@ def extend(vm, background, all_, filter):
 
     :param filter: a filter string matching broker's specification
     """
+    broker_args = helpers.clean_dict(kwargs)
     if background:
         helpers.fork_broker()
     inventory = helpers.load_inventory(filter=filter)
@@ -250,7 +269,7 @@ def extend(vm, background, all_, filter):
     for num, host in enumerate(inventory):
         if str(num) in vm or host["hostname"] in vm or host["name"] in vm or all_:
             to_extend.append(VMBroker().reconstruct_host(host))
-    broker_inst = VMBroker(hosts=to_extend)
+    broker_inst = VMBroker(hosts=to_extend, **broker_args)
     broker_inst.extend()
 
 
@@ -355,6 +374,7 @@ def execute(ctx, background, nick, output_format, artifacts, args_file, **kwargs
         helpers.fork_broker()
     broker_inst = VMBroker(**broker_args)
     result = broker_inst.execute()
+    helpers.emit({"output": result})
     if output_format == "raw":
         print(result)
     elif output_format == "log":
