@@ -27,8 +27,10 @@ class AnsibleTower(Provider):
     _validators = [
         Validator("ANSIBLETOWER.release_workflow", default="remove-vm"),
         Validator("ANSIBLETOWER.extend_workflow", default="extend-vm"),
+        Validator("ANSIBLETOWER.new_expire_time", default="+172800"),
         Validator("ANSIBLETOWER.workflow_timeout", is_type_of=int, default=3600),
         Validator("ANSIBLETOWER.results_limit", is_type_of=int, default=20),
+        Validator("ANSIBLETOWER.error_scope", default="last"),
         Validator("ANSIBLETOWER.base_url", must_exist=True),
         # Validator combination for username+password or token
         (
@@ -62,6 +64,13 @@ class AnsibleTower(Provider):
             "--job-template", type=str, help="Name of a job template to execute"
         ),
     ]
+    _extend_options = [
+        click.option(
+            "--new-expire-time",
+            type=str,
+            help="Time host should expire or time added to host reservation.",
+        ),
+    ]
 
     def __init__(self, **kwargs):
         # Get the specified AT instance
@@ -89,6 +98,7 @@ class AnsibleTower(Provider):
         # https://github.com/ansible/awx/blob/ddb6c5d0cce60779be279b702a15a2fddfcd0724/awxkit/awxkit/cli/client.py#L85-L94
         # unit test mock structure means the root API instance can't be loaded on the same line
         if self.token:
+            helpers.emit(auth_type="token")
             logger.info("Using token authentication")
             config.token = self.token
             try:
@@ -111,6 +121,7 @@ class AnsibleTower(Provider):
                     message="Failed to lookup a username for the given token, please check credentials",
                 )
         else:  # dynaconf validators should have checked that either token or password was provided
+            helpers.emit(auth_type="password")
             logger.info("Using username and password authentication")
             config.credentials = {
                 "default": {"username": self.uname, "password": self.pword}
@@ -207,6 +218,44 @@ class AnsibleTower(Provider):
                         )
         return artifacts
 
+    def _get_failure_messages(self, workflow):
+        """Find all failure nodes and aggregate failure messages"""
+        failure_messages = []
+        # get all failed job nodes (iterate)
+        if "workflow_nodes" in workflow.related:
+            children = workflow.get_related("workflow_nodes").results
+            # filter out children with no associated job
+            children = list(
+                filter(
+                    lambda child: getattr(child.summary_fields, "job", None), children
+                )
+            )
+            # filter out children that didn't fail
+            children = list(
+                filter(
+                    lambda child: child.summary_fields.job.failed, children
+                )
+            )
+            children.sort(key=lambda child: child.summary_fields.job.id)
+            for child in children[::-1]:
+                if child.type == "workflow_job_node":
+                    logger.debug(child)
+                    child_id = child.summary_fields.job.id
+                    child_obj = self.v2.jobs.get(id=child_id).results
+                    if child_obj:
+                        child_obj = child_obj.pop()
+                        # get all failed job_events for each job (filter failed=true)
+                        failed_events = [ev for ev in child_obj.get_related("job_events", page_size=200).results if ev.failed]
+                        # find the one(s) with event_data['res']['msg']
+                        failure_messages.extend([
+                            {"job": child_obj.name, "task": ev.event_data["play"], "reason": ev.event_data["res"]["msg"]}
+                            for ev in failed_events if ev.event_data.get("res")
+                        ])
+        if settings.ANSIBLETOWER.error_scope == "last":
+            return failure_messages[0]
+        else:
+            return failure_messages
+
     def _get_expire_date(self, host_id):
         try:
             time_stamp = (
@@ -230,25 +279,33 @@ class AnsibleTower(Provider):
         expire_time = self._get_expire_date(host.id)
         if expire_time:
             host_info["expire_time"] = expire_time
-        if "last_job" in host.related:
-            last_job = host.get_related("last_job")
-            job_vars = json.loads(last_job.extra_vars)
-            host_info["_broker_args"].update(
-                {
-                    arg: val
-                    for arg, val in job_vars.items()
-                    if val and isinstance(val, str)
-                }
-            )
-            host_info["_broker_args"]["tower_inventory"] = last_job.inventory
-            try:
-                host_info["_broker_args"]["workflow"] = host.get_related(
-                    "last_job"
-                ).summary_fields.source_workflow_job.name
-            except:
-                logger.warning(
-                    f"Unable to determine workflow for {host_info['hostname']}"
-                )
+        try:
+            create_job = self.v2.jobs.get(id=host.get_related("job_events").results[0].job)
+            create_job = create_job.results[0].get_related("source_workflow_job")
+            host_info["_broker_args"]["workflow"] = create_job.name
+        except IndexError:
+            if "last_job" in host.related:
+                # potentially not create job, but easier processing below
+                create_job = host.get_related("last_job")
+                try:
+                    host_info["_broker_args"]["workflow"] = host.get_related(
+                        "last_job"
+                    ).summary_fields.source_workflow_job.name
+                except:
+                    logger.warning(
+                        f"Unable to determine workflow for {host_info['hostname']}"
+                    )
+            else:
+                return host_info
+        create_vars = json.loads(create_job.extra_vars)
+        host_info["_broker_args"].update(
+            {
+                arg: val
+                for arg, val in create_vars.items()
+                if val and isinstance(val, str)
+            }
+        )
+        host_info["_broker_args"]["tower_inventory"] = create_job.inventory
         return host_info
 
     @cached_property
@@ -357,21 +414,25 @@ class AnsibleTower(Provider):
         )
         job = target.launch(payload=payload)
         job_number = job.url.rstrip("/").split("/")[-1]
+        job_api_url = url_parser.urljoin(self.url, str(job.url))
         job_ui_url = url_parser.urljoin(self.url, f"/#/{subject}s/{job_number}")
+        helpers.emit(api_url=job_api_url, ui_url=job_ui_url)
         logger.info(
             "Waiting for job: \n"
-            f"API: {url_parser.urljoin(self.url, str(job.url))}\n"
+            f"API: {job_api_url}\n"
             f"UI: {job_ui_url}"
         )
         job.wait_until_completed(timeout=settings.ANSIBLETOWER.workflow_timeout)
         if not job.status == "successful":
+            message_data = {
+                f"{subject.capitalize()} Status": job.status,
+                "Reason(s)": self._get_failure_messages(job),
+                "URL": job_ui_url
+            }
+            helpers.emit(message_data)
             raise exceptions.ProviderError(
                 provider="AnsibleTower",
-                message=(
-                    f"{subject.capitalize()} Status: {job.status}\n"
-                    f"Explanation: {job.job_explanation}\n"
-                    f"UI: {job_ui_url}"
-                ),
+                message=message_data["Reason(s)"]
             )
         if (artifacts := kwargs.get("artifacts")) :
             del kwargs["artifacts"]
@@ -392,7 +453,7 @@ class AnsibleTower(Provider):
             hosts.extend(inv_hosts)
         return [self._compile_host_info(host) for host in hosts]
 
-    def extend_vm(self, target_vm):
+    def extend_vm(self, target_vm, new_expire_time=None):
         """Run the extend workflow with defaults args
 
         :param target_vm: This should be a host object
@@ -403,7 +464,10 @@ class AnsibleTower(Provider):
                 self._inventory = new_inv
                 del self.inventory  # clear the cached value
         return self.execute(
-            workflow=settings.ANSIBLETOWER.extend_workflow, target_vm=target_vm.name
+            workflow=settings.ANSIBLETOWER.extend_workflow,
+            target_vm=target_vm.name,
+            new_expire_time=new_expire_time
+            or settings.ANSIBLETOWER.get("new_expire_time"),
         )
 
     def nick_help(self, **kwargs):
@@ -418,6 +482,7 @@ class AnsibleTower(Provider):
             workflows = [
                 workflow.name
                 for workflow in self.v2.workflow_job_templates.get().results
+                if workflow.summary_fields.user_capabilities.get("start")
             ]
             if (res_filter := kwargs.get("results_filter")) :
                 workflows = results_filter(workflows, res_filter)
@@ -445,6 +510,7 @@ class AnsibleTower(Provider):
             job_templates = [
                 job_template.name
                 for job_template in self.v2.job_templates.get().results
+                if job_template.summary_fields.user_capabilities.get("start")
             ]
             if (res_filter := kwargs.get("results_filter")) :
                 job_templates = results_filter(job_templates, res_filter)
