@@ -1,5 +1,6 @@
 import os
 import socket
+from pathlib import Path
 from ssh2.session import Session as ssh2_Session
 from ssh2 import sftp as ssh2_sftp
 
@@ -70,11 +71,16 @@ class Session:
         """read a remote file into a local destination"""
         if not destination:
             destination = source
+        # create the destination path if it doesn't exist
+        destination = Path(destination)
+        destination.parent.mkdir(parents=True, exist_ok=True)
+        destination.touch()
+        # initiate the sftp session, read data, write it to a local destination
         sftp = self.session.sftp_init()
         with sftp.open(
             source, ssh2_sftp.LIBSSH2_FXF_READ, ssh2_sftp.LIBSSH2_SFTP_S_IRUSR
         ) as remote:
-            with open(destination, "wb") as local:
+            with destination.open("wb") as local:
                 for size, data in remote:
                     local.write(data)
 
@@ -88,12 +94,23 @@ class Session:
                 for data in local:
                     remote.write(data)
 
+    def remote_copy(self, source, dest_host):
+        """Copy a file from this host to another"""
+        sftp_down = self.session.sftp_init()
+        sftp_up = dest_host.session.session.sftp_init()
+        with sftp_down.open(
+            source, ssh2_sftp.LIBSSH2_FXF_READ, ssh2_sftp.LIBSSH2_SFTP_S_IRUSR
+        ) as download:
+            with sftp_up.open(source, FILE_FLAGS, SFTP_MODE) as upload:
+                for size, data in download:
+                    upload.write(data)
+
     def scp_write(self, source, destination=None):
         """scp write a local file to a remote destination"""
         if not destination:
             destination = source
-        fileinfo = os.stat(args.source)
-        chan = s.scp_send64(
+        fileinfo = os.stat(source)
+        chan = self.session.scp_send64(
             destination,
             fileinfo.st_mode & 0o777,
             fileinfo.st_size,
